@@ -118,7 +118,12 @@ _add_address()
 		set -- "${one}/$(_netmask2cidr "${three}")" "$@"
 	fi
 	
-	#config=( "${config[@]//pointopoint/peer}" )
+	# tunnel keyword is 'peer' in iproute2, but 'pointopoint' in ifconfig.
+	if [ "$2" = "pointopoint" ]; then
+		local one="$1"
+		shift; shift
+		set -- "${one}" "peer" "$@"
+	fi
 	
 	# Always scope lo addresses as host unless specified otherwise
 	if [ "${IFACE}" = "lo" ]; then
@@ -136,6 +141,7 @@ _add_address()
 			;;
 	esac
 
+	veinfo ip addr add "$@" dev "${IFACE}"
 	ip addr add "$@" dev "${IFACE}"
 }
 
@@ -175,6 +181,7 @@ _add_route()
 		cmd="${cmd} metric ${metric}"
 	fi
 
+	veinfo ip ${family} route append ${cmd} dev "${IFACE}"
 	ip ${family} route append ${cmd} dev "${IFACE}"
 	eend $?
 }
@@ -199,6 +206,32 @@ _tunnel()
 	ip tunnel "$@"
 }
 
+# This is just to trim whitespace, do not add any quoting!
+_trim() {
+	echo $*
+}
+
+# This is our interface to Routing Policy Database RPDB
+# This allows for advanced routing tricks
+_ip_rule_runner() {
+	local cmd rules OIFS="${IFS}"
+	cmd="$1"
+	rules="$2"
+	veindent
+	local IFS="$__IFS"
+	for ru in $rules ; do
+		unset IFS
+		ruN="$(trim "${ru}")"
+		[ -z "${ruN}" ] && continue
+		vebegin "${cmd} ${ruN}"
+		ip rule ${cmd} ${ru}
+		veend $?
+		local IFS="$__IFS"
+	done
+	IFS="${OIFS}"
+	veoutdent
+}
+
 iproute2_pre_start()
 {
 	local tunnel=
@@ -210,7 +243,7 @@ iproute2_pre_start()
 		ebegin "Creating tunnel ${IFVAR}"
 		ip tunnel add ${tunnel} name "${IFACE}"
 		eend $? || return 1
-		_up	
+		_up
 	fi
 
 	# MTU support
@@ -240,6 +273,16 @@ iproute2_post_start()
 
 	# Kernel may not have IP built in
 	if [ -e /proc/net/route ]; then
+		local rules="$(_get_array "rules_${IFVAR}")"
+		if [ -n "${rules}" ]; then
+			if ! ip rule list | grep -q "^"; then
+				eerror "IP Policy Routing (CONFIG_IP_MULTIPLE_TABLES) needed for ip rule"
+			else
+				service_set_value "ip_rule" "${rules}"
+				einfo "Adding RPDB rules"
+				_ip_rule_runner add "${rules}"
+			fi
+		fi
 		ip route flush table cache dev "${IFACE}"
 	fi
 
@@ -259,6 +302,16 @@ iproute2_post_start()
 
 iproute2_post_stop()
 {
+	# Kernel may not have IP built in
+	if [ -e /proc/net/route ]; then
+		local rules="$(service_get_value "ip_rule")"
+		if [ -n "${rules}" ]; then
+			einfo "Removing RPDB rules"
+			_ip_rule_runner del "${rules}"
+		fi
+		ip route flush table cache dev "${IFACE}"
+	fi
+
 	# Don't delete sit0 as it's a special tunnel
 	if [ "${IFACE}" != "sit0" ]; then
 		if [ -n "$(ip tunnel show "${IFACE}" 2>/dev/null)" ]; then
