@@ -878,9 +878,16 @@ main(int argc, char **argv)
 			eerrorx("%s: %s", applet, strerror(errno));
 			/* NOTREACHED */
 		case 'S':
-			bootlevel = rc_sys();
-			if (bootlevel)
-				printf("%s\n", bootlevel);
+			if (rc_conf_value("rc_sys")) {
+				bootlevel = rc_sys_v2();
+				if(bootlevel)
+					printf("%s\n", bootlevel);
+			} else {
+				ewarn("WARNING: rc_sys not defined in rc.conf. Falling back to automatic detection");
+				bootlevel = rc_sys_v1();
+				if(bootlevel)
+					printf("%s\n", bootlevel);
+			}
 			exit(EXIT_SUCCESS);
 			/* NOTREACHED */
 			case_RC_COMMON_GETOPT
